@@ -37,9 +37,3 @@
 
 // Error code to return if argument parsing fails (default 1)
 #define TOYFLAG_ARGFAIL(x) (x<<24)
-
-#if CFG_TOYBOX_PEDANTIC_ARGS
-#define NO_ARGS ">0"
-#else
-#define NO_ARGS 0
-#endif
