@@ -3,6 +3,9 @@
  * Copyright 2015 Rob Landley <rob@landley.net>
  *
  * No standard.
+ *
+ * In 1979 3BSD's tset had a sleep(1) to let mechanical printer-and-ink
+ * terminals "settle down". We're not doing that.
 
 USE_RESET(NEWTOY(reset, 0, TOYFLAG_USR|TOYFLAG_BIN))
 
@@ -22,5 +25,5 @@ void reset_main(void)
 
   // man 4 console_codes: reset terminal is ESC (no left bracket) c
   // DEC private mode set enable wraparound sequence.
-  xwrite(fd<0 ? 1 : fd, "\033c\033[?7h", 2);
+  xwrite(fd<0 ? 1 : fd, "\ec\e[?7h", 2);
 }
