@@ -24,7 +24,10 @@ config SLEEP
 void sleep_main(void)
 {
   struct timespec ts;
+  char **args;
 
-  xparsetimespec(*toys.optargs, &ts);
-  toys.exitval = !!nanosleep(&ts, NULL);
+  for (args = toys.optargs; !toys.exitval && *args; args++) {
+    xparsetimespec(*args, &ts);
+    toys.exitval = !!nanosleep(&ts, NULL);
+  }
 }
